import { useId, useQueryParams } from '@showroomjs/ui';
import lzString from 'lz-string';
import * as React from 'react';
const commentReducer = (state, event) => {
    switch (event.type) {
        case 'add':
            return Object.assign(Object.assign({}, state), { items: state.items.concat(event.data) });
        case 'set':
            return {
                items: event.items,
            };
        case 'update':
            return Object.assign(Object.assign({}, state), { items: state.items.map((item) => item.id === event.data.id
                    ? Object.assign(Object.assign({}, item), event.data) : item) });
        case 'remove':
            return Object.assign(Object.assign({}, state), { items: state.items.filter((item) => item.id !== event.id) });
        case 'clear':
            return defaultState;
        default:
            throw new Error(`Unsupported event type: ${event.type}`);
    }
};
const defaultState = {
    items: [],
};
export const useCommentState = (idPrefix) => {
    const id_prefix = useId(idPrefix);
    const idCountRef = React.useRef(0);
    const [state, dispatch] = React.useReducer(commentReducer, defaultState);
    const [qParams, setQParams] = useQueryParams();
    React.useEffect(() => {
        const commentQuery = qParams.get('comments');
        if (commentQuery) {
            const stored = deCompress(commentQuery);
            if (stored) {
                dispatch({
                    type: 'set',
                    items: stored,
                });
            }
        }
    }, []);
    React.useEffect(() => {
        setQParams({
            comments: state.items.length > 0 ? compress(state.items) : undefined,
        }, {
            merge: true,
        });
    }, [state.items]);
    return {
        state,
        add: React.useCallback((data) => {
            dispatch({
                type: 'add',
                data: Object.assign(Object.assign({}, data), { id: `${id_prefix}-${Date.now()}-${idCountRef.current++}` }),
            });
        }, []),
        update: React.useCallback((changes) => {
            dispatch({
                type: 'update',
                data: changes,
            });
        }, []),
        remove: React.useCallback((id) => {
            dispatch({
                type: 'remove',
                id,
            });
        }, []),
        clear: React.useCallback(() => dispatch({
            type: 'clear',
        }), []),
    };
};
const compress = (data) => {
    return lzString.compressToEncodedURIComponent(JSON.stringify(data));
};
const deCompress = (str) => {
    try {
        const decompressed = lzString.decompressFromEncodedURIComponent(str);
        if (!decompressed) {
            return null;
        }
        const parsedResult = JSON.parse(decompressed);
        return Array.isArray(parsedResult) ? parsedResult : null;
    }
    catch (err) {
        return null;
    }
};
//# sourceMappingURL=use-comment-state.js.map