"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateReactEntryCompat = exports.generateSearchIndex = exports.generateDocPlaceHolder = exports.generateWrapper = exports.generateAllComponentsDocs = exports.generateAllComponentsPaths = exports.generateAllComponents = exports.generateSectionsAndImports = exports.generateCodeblocksData = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@showroomjs/core");
const path_1 = tslib_1.__importDefault(require("path"));
const log_to_stdout_1 = require("./log-to-stdout");
const paths_1 = require("./paths");
let _nameIndex = 0;
const getName = (name) => (0, core_1.getSafeName)(name) + '_' + _nameIndex++;
const generateCodeblocksData = (sections) => {
    const codeBlocksImportPaths = [];
    function collectCodeblocks(section) {
        switch (section.type) {
            case 'component':
                if (section.docPath) {
                    codeBlocksImportPaths.push(`${section.docPath}?showroomRemarkCodeblocks`);
                }
                break;
            case 'markdown': {
                if (section._associatedComponentName) {
                    codeBlocksImportPaths.push(`${section.sourcePath}?showroomRemarkCodeblocks`);
                }
                else {
                    codeBlocksImportPaths.push(`${section.sourcePath}?showroomRemarkDocCodeblocks`);
                }
                break;
            }
            case 'group':
                section.items.forEach(collectCodeblocks);
                break;
            default:
                break;
        }
    }
    sections.forEach(collectCodeblocks);
    return `${codeBlocksImportPaths
        .map((path, index) => `import block${index} from '${path}';`)
        .join('\n')}
  export default {
        items: [
            ${codeBlocksImportPaths
        .map((_, index) => `block${index}`)
        .join(',')}
        ],
    }`;
};
exports.generateCodeblocksData = generateCodeblocksData;
function compileComponentSection(component, { metadataIdentifier, enableAdvancedEditor, getEditUrl, }) {
    const { docPath, sourcePath } = component;
    const { name: componentName } = path_1.default.parse(sourcePath);
    const editUrl = getEditUrl && docPath
        ? getEditUrl({
            relativePath: path_1.default.relative(paths_1.paths.appPath, docPath),
        })
        : null;
    const load = docPath
        ? `async () => {
  const loadDoc = import(/* webpackChunkName: "${componentName}-doc" */'${docPath}');
  const loadImports = import(/* webpackChunkName: "${componentName}-imports" */'${docPath}?showroomRemarkImports');
  const loadCodeBlocks = import(/* webpackChunkName: "${componentName}-codeblocks" */'${docPath}?showroomRemarkCodeblocks');
  const Component = await import(/* webpackChunkName: "${componentName}" */'${sourcePath}');

  const { default: doc, headings } = await loadDoc;
  const { imports } = await loadImports;

  return {
    doc,
    headings,
    Component: Component.default || Component[${metadataIdentifier}.displayName] || Component,
    imports: imports || {},
    codeblocks: (await loadCodeBlocks).default || {},
    editUrl: ${JSON.stringify(editUrl)},
    loadDts: () => ${enableAdvancedEditor
            ? `import('${docPath}?showroomRemarkImportsDts')`
            : `Promise.resolve({default: {}})`},
  }    
}`
        : `async () => {
      const Component = await import(/* webpackChunkName: "${componentName}" */'${sourcePath}');

      return {
        Component: Component.default || Component[${metadataIdentifier}.displayName] || Component,
        doc: null,
        headings: [],
        imports: {},
        codeblocks: {},
        loadDts: () => Promise.resolve({ default: {} }),
      }
    }`;
    return `{
      load: ${load},
    }`;
}
const generateSectionsAndImports = (sections, options) => {
    const imports = [];
    const codeImportImports = [];
    const compVar = [];
    function collect(sectionList) {
        return `[${sectionList
            .map((section) => {
            if (section.type === 'group') {
                return `{
            type: 'group',
            title: '${section.title}',
            items: ${collect(section.items)},
            slug: addTrailingSlash('${section.slug}'),
            ${section.hideFromSidebar ? 'hideFromSidebar: true,' : ''}
          }`;
            }
            if (section.type === 'component') {
                const name = getName('component');
                if (section.docPath) {
                    codeImportImports.push({
                        name,
                        path: `${section.docPath}?showroomRemarkImports`,
                    });
                }
                compVar.push({
                    identifier: name,
                    sourcePath: section.sourcePath,
                });
                return `${name} ? {
              type: 'component',
              data: ${compileComponentSection(section, {
                    enableAdvancedEditor: options.enableAdvancedEditor,
                    getEditUrl: options.getEditUrl,
                    metadataIdentifier: name,
                })},
              metadata: ${name},
              title: ${name}.displayName,
              description: ${name}.description,
              slug: addTrailingSlash([${section.parentSlugs.length > 0
                    ? `'${section.parentSlugs.join('/')}',`
                    : ''} slugify(${name}.displayName, { lower: true })].join('/')),
              id: '${section.id}',
              ${section.hideFromSidebar ? 'hideFromSidebar: true,' : ''}
              shouldIgnore: ${!!options.skipEmptyComponent} && (${!section.docPath} && !${name}.description && Object.keys(${name}.props).length === 0),
            } : {
              shouldIgnore: true,
            }`;
                // because the component parsing may return nothing, so need to set a flag here and filter it at bottom
            }
            if (section.type === 'markdown') {
                const name = getName('markdown');
                imports.push({
                    type: 'default',
                    name: `${name}_frontmatter`,
                    path: `${section.sourcePath}?showroomFrontmatter`,
                });
                const isTreatedAsComponentDoc = !!section._associatedComponentName;
                codeImportImports.push({
                    name,
                    path: `${section.sourcePath}?${isTreatedAsComponentDoc
                        ? 'showroomRemarkImports'
                        : 'showroomRemarkDocImports'}`,
                });
                const editUrl = options.getEditUrl && section.sourcePath
                    ? options.getEditUrl({
                        relativePath: path_1.default.relative(paths_1.paths.appPath, section.sourcePath),
                    })
                    : null;
                const chunkName = `${section.title || section.slug || ''}${name}`;
                return `{
              type: 'markdown',
              fallbackTitle: '${section.title || ''}',
              slug: addTrailingSlash('${section.slug}'),
              frontmatter: ${name}_frontmatter || {},
              ${section.hideFromSidebar ? 'hideFromSidebar: true,' : ''}
              ${isTreatedAsComponentDoc
                    ? `_associatedComponentName: '${section._associatedComponentName}',`
                    : ''}
              formatLabel: ${section.formatLabel.toString()},
              load: async () => {
                const loadComponent = import(/* webpackChunkName: "${chunkName}" */'${section.sourcePath}');
                const loadImports = import(/* webpackChunkName: "${chunkName}-imports" */'${section.sourcePath}?${isTreatedAsComponentDoc
                    ? 'showroomRemarkImports'
                    : 'showroomRemarkDocImports'}');
                const loadCodeblocks = import(/* webpackChunkName: "${chunkName}-codeblocks" */'${section.sourcePath}?${isTreatedAsComponentDoc
                    ? 'showroomRemarkCodeblocks'
                    : 'showroomRemarkDocCodeblocks'}');

                const { default: Component, headings } = await loadComponent;
                const { imports = {} } = await loadImports;

                return {
                  Component,
                  headings,
                  editUrl: ${JSON.stringify(editUrl)},
                  imports,
                  codeblocks: (await loadCodeblocks).default || {},
                  loadDts: () => ${options.enableAdvancedEditor
                    ? `import('${section.sourcePath}?${isTreatedAsComponentDoc
                        ? 'showroomRemarkImportsDts'
                        : 'showroomRemarkDocImportsDts'}')`
                    : 'Promise.resolve({ default: {} })'},
                }
              },
            }`;
            }
            return JSON.stringify(section);
        })
            .join(', ')}].filter(el => !el.shouldIgnore)`;
    }
    const result = collect(sections);
    return {
        sections: `import slugify from 'slugify';
    import allCompMetadata from 'react-showroom-all-components-docs';
    import { addTrailingSlash } from '@showroomjs/core';
    ${imports
            .map((imp) => imp.type === 'default'
            ? `import ${imp.name} from '${imp.path}';`
            : `import * as ${imp.name} from '${imp.path}';`)
            .join('\n')}
    ${compVar
            .map((varDef) => `const ${varDef.identifier} = allCompMetadata['${varDef.sourcePath}'];`)
            .join('\n')}
    export default ${result}`,
        allImports: `${codeImportImports
            .map((imp) => `import * as ${imp.name} from '${imp.path}';`)
            .join('\n')}
    export default Object.assign({}, ${codeImportImports
            .map((imp) => `${imp.name}.imports`)
            .join(', ')});`,
    };
};
exports.generateSectionsAndImports = generateSectionsAndImports;
const generateAllComponents = (sections) => {
    const componentImports = [];
    function collect(sectionList) {
        sectionList.forEach((section) => {
            if (section.type === 'group') {
                collect(section.items);
            }
            if (section.type === 'component') {
                const name = getName('component');
                componentImports.push({
                    name,
                    sourcePath: section.sourcePath,
                    codeblockPath: section.docPath && `${section.docPath}?showroomRemarkCodeblocks`,
                });
            }
        });
    }
    collect(sections);
    if (componentImports.length === 0) {
        return `export const AllComponents = {};`;
    }
    return `import allCompMetadata from 'react-showroom-all-components-docs';
  ${componentImports
        .map((comp) => `const ${comp.name} = require('${comp.sourcePath}');
  const _showroomMetadata_${comp.name} = allCompMetadata['${comp.sourcePath}'];
  ${comp.codeblockPath
        ? `import _code_${comp.name} from '${comp.codeblockPath}';`
        : ''}`)
        .join('\n')}

  export const AllComponents = Object.assign(
    ${componentImports
        .map((comp) => {
        const compVar = comp.name;
        const metadataVar = `_showroomMetadata_${comp.name}`;
        return `${metadataVar} ? {
      [${metadataVar}.displayName]: ${compVar}.default || ${compVar}[${metadataVar}.displayName] || ${compVar}
    } : {}`;
    })
        .join(', ')}
  );`;
};
exports.generateAllComponents = generateAllComponents;
const generateAllComponentsPaths = (sections) => {
    const result = [];
    function collect(sectionList) {
        sectionList.forEach((section) => {
            if (section.type === 'group') {
                collect(section.items);
            }
            if (section.type === 'component') {
                result.push(section.sourcePath);
            }
        });
    }
    collect(sections);
    return JSON.stringify(result, null, 2);
};
exports.generateAllComponentsPaths = generateAllComponentsPaths;
const generateAllComponentsDocs = (sections) => {
    const componentPaths = [];
    function collect(sectionList) {
        sectionList.forEach((section) => {
            if (section.type === 'group') {
                collect(section.items);
            }
            if (section.type === 'component') {
                componentPaths.push(section.sourcePath);
            }
        });
    }
    collect(sections);
    return `${componentPaths
        .map((path, index) => `import component${index} from '${path}?docgen';`)
        .join('\n')}

  const allComponentDocs = {};

  ${componentPaths
        .map((_, index) => `if (component${index}) {
    allComponentDocs[component${index}.filePath] = component${index};
  }`)
        .join('\n')}
  
  export default allComponentDocs;`;
};
exports.generateAllComponentsDocs = generateAllComponentsDocs;
const generateWrapper = (wrapper) => {
    if (wrapper) {
        return `import Wrapper from '${wrapper}';
      export default Wrapper`;
    }
    return `import * as React from 'react';
    export default React.Fragment;`;
};
exports.generateWrapper = generateWrapper;
const generateDocPlaceHolder = (placeholder) => {
    if (placeholder) {
        return `import Placeholder from '${placeholder}';
    export default Placeholder`;
    }
    return `import { DocPlaceholder } from '${(0, paths_1.resolveShowroom)('client-dist/components/doc-placeholder.js')}';
  export default DocPlaceholder`;
};
exports.generateDocPlaceHolder = generateDocPlaceHolder;
const generateSearchIndex = (sections, includeHeadingsInIndex) => {
    const compVar = [];
    const imports = [];
    function collect(sectionList) {
        return `[${sectionList
            .map((section) => {
            if (section.hideFromSidebar) {
                return 'undefined';
            }
            if (section.type === 'group') {
                return `{ type: 'group', items: ${collect(section.items)} }`;
            }
            if (section.type === 'component') {
                const name = getName('component');
                compVar.push({
                    identifier: name,
                    sourcePath: section.sourcePath,
                });
                const shouldIndexDoc = includeHeadingsInIndex && !!section.docPath;
                if (shouldIndexDoc) {
                    imports.push({
                        type: 'default',
                        name: `${name}_doc`,
                        path: `${section.docPath}?headings`,
                    });
                }
                return `${name} && {
          type: 'component',
          metadata: ${name},
          title: ${name}.displayName,
          description: ${name}.description,
          slug: [${section.parentSlugs.length > 0
                    ? `'${section.parentSlugs.join('/')}',`
                    : ''} slugify(${name}.displayName, { lower: true })].join('/'),
          id: '${section.id}',
          ${shouldIndexDoc ? `headings: ${name}_doc,` : ''}
        }`;
            }
            if (section.type === 'markdown') {
                const name = getName('markdown');
                const mainData = {
                    type: 'default',
                    name: `${name}_frontmatter`,
                    path: `${section.sourcePath}?showroomFrontmatter`,
                };
                if (includeHeadingsInIndex) {
                    imports.push({
                        type: 'default',
                        name: name,
                        path: `${section.sourcePath}?headings`,
                    }, mainData);
                }
                else {
                    imports.push(mainData);
                }
                return `{
            type: 'markdown',
            slug: '${section.slug}',
            frontmatter: ${name}_frontmatter || {},
            fallbackTitle: '${section.title || ''}',
            ${includeHeadingsInIndex ? `headings: ${name},` : ''}
            formatLabel: ${section.formatLabel.toString()},
          }`;
            }
            if (section.type === 'link') {
                return `{ type: 'link', href: '${section.href}', title: '${section.title}' }`;
            }
        })
            .join(', ')}].filter(Boolean)`;
    }
    const result = collect(sections);
    return `import slugify from 'slugify';
import allCompMetadata from 'react-showroom-all-components-docs';
${imports
        .map((imp) => imp.type === 'default'
        ? `import ${imp.name} from '${imp.path}';`
        : `import * as ${imp.name} from '${imp.path}';`)
        .join('\n')}
${compVar
        .map((varDef) => `const ${varDef.identifier} = allCompMetadata['${varDef.sourcePath}'];`)
        .join('\n')}
export default ${result};`;
};
exports.generateSearchIndex = generateSearchIndex;
const generateReactEntryCompat = () => {
    const ReactDomPath = require.resolve('react-dom', {
        paths: [paths_1.paths.appPath],
    });
    const ReactDOM = require(ReactDomPath);
    const reactMajorVer = Number(ReactDOM.version.split('.')[0]);
    (0, log_to_stdout_1.logToStdout)(`Using React version ${(0, log_to_stdout_1.yellow)(reactMajorVer)}`);
    if (reactMajorVer <= 17) {
        return `import * as ReactDOM from 'react-dom';
    
    export const render = (ui, target) => {
      ReactDOM.render(ui, target);

      return function unmount() {
        ReactDOM.unmountComponentAtNode(target);
      }
    };

    export const hydrate = (ui, target) => {
      ReactDOM.hydrate(ui, target);

      return function unmount() {
        ReactDOM.unmountComponentAtNode(target);
      };
    };
`;
    }
    else {
        return `import { createRoot, hydrateRoot } from 'react-dom/client';
    
    export const render = (ui, target) => {
      const root = createRoot(target);

      root.render(ui);

      return function unmount() {
        root.unmount();
      }
    };

    export const hydrate = (ui, target) => {
      const root = hydrateRoot(target, ui);

      return function unmount() {
        root.unmount();
      };
    };
`;
    }
};
exports.generateReactEntryCompat = generateReactEntryCompat;
//# sourceMappingURL=generate-showroom-data.js.map