"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@showroomjs/core");
const fs = tslib_1.__importStar(require("fs"));
const glob = tslib_1.__importStar(require("glob"));
const mimeTypes = tslib_1.__importStar(require("mime-types"));
const path = tslib_1.__importStar(require("path"));
const nightOwl_1 = tslib_1.__importDefault(require("prism-react-renderer/themes/nightOwl"));
const slugify_1 = tslib_1.__importDefault(require("slugify"));
const ts = tslib_1.__importStar(require("typescript"));
const create_hash_1 = require("./create-hash");
const log_to_stdout_1 = require("./log-to-stdout");
const paths_1 = require("./paths");
const DEFAULT_COMPONENTS_GLOB = 'src/components/**/*.tsx';
const DEFAULT_IGNORES = [
    '**/__tests__/**',
    '**/*.test.{ts,tsx}',
    '**/*.spec.{ts,tsx}',
    '**/*.d.ts',
];
const defaultConfig = {
    basePath: '',
    url: '',
    codeTheme: nightOwl_1.default,
    resetCss: true,
    outDir: 'showroom',
};
const docgenOptions = {
    propFilter: (prop) => {
        if (prop.parent) {
            return !prop.parent.fileName.includes('@types/react');
        }
        return true;
    },
    shouldExtractLiteralValuesFromEnum: true,
    shouldExtractValuesFromUnion: true,
    shouldRemoveUndefinedFromOptional: true,
    shouldIncludePropTagMap: true,
};
const defaultThemeConfiguration = {
    title: 'React Showroom',
    codeTheme: nightOwl_1.default,
    resetCss: true,
    audienceToggle: 'design',
    navbar: {},
    colors: {
        'primary-50': '#FDF2F8',
        'primary-100': '#FCE7F3',
        'primary-200': '#FBCFE8',
        'primary-300': '#F9A8D4',
        'primary-400': '#F472B6',
        'primary-500': '#EC4899',
        'primary-600': '#DB2777',
        'primary-700': '#BE185D',
        'primary-800': '#9D174D',
        'primary-900': '#831843',
    },
    serviceWorker: true,
};
const deviceDevices = [
    'iPhone 6/7/8',
    'iPad',
    'Macbook Air',
];
let _normalizedConfig;
const getConfig = (env, options) => {
    if (_normalizedConfig) {
        return _normalizedConfig;
    }
    const { configFile, userConfig } = options;
    const { build: providedBuildConfig = {}, devServer: providedDevServerConfig = {}, components: providedComponentGlob, items, docgen: providedDocgenConfig = {}, theme: { manifest: providedManifestConfig = {}, ...providedThemeConfig } = {}, imports: providedImports, ignores = DEFAULT_IGNORES, cacheDir = '.showroom_cache', componentsEntry, sass = isSassInstalled(), css = {
        postcss: fs.existsSync(paths_1.paths.appPostcssConfig),
    }, example: { widths, dimensions = widths
        ? widths.map((width) => ({
            width: width,
            height: '100%',
            name: `${width}px`,
        }))
        : deviceDevices, placeholder, enableAdvancedEditor = true, showDeviceFrame = !widths, syncStateType = 'state', a11y = {
        config: {},
    }, } = {}, html = {}, search: { includeHeadings: searchIncludeHeadings = env === 'production', } = {}, ...providedConfig } = userConfig || getUserConfig(env, configFile);
    const sections = [];
    const components = [];
    if (providedComponentGlob) {
        const componentPaths = glob.sync(providedComponentGlob, {
            cwd: paths_1.paths.appPath,
            absolute: true,
            ignore: ignores,
        });
        collectComponents(componentPaths, sections, [], false);
    }
    else if (!items) {
        const componentPaths = glob.sync(DEFAULT_COMPONENTS_GLOB, {
            cwd: paths_1.paths.appPath,
            absolute: true,
            ignore: ignores,
        });
        collectComponents(componentPaths, sections, [], false);
    }
    if (items) {
        collectSections(items, sections, []);
    }
    if (!sections.some((section) => 'slug' in section && section.slug === '')) {
        // use README.md as home page if no home page
        const readmePath = path.resolve(paths_1.paths.appPath, 'README.md');
        if (fs.existsSync(readmePath)) {
            sections.push({
                type: 'markdown',
                sourcePath: readmePath,
                slug: '',
                formatLabel: (x) => x,
            });
        }
    }
    const { outDir = 'showroom', prerender = true } = providedBuildConfig;
    const imports = providedImports
        ? providedImports.some((imp) => imp === 'react')
            ? providedImports
            : providedImports.concat('react')
        : ['react'];
    const tsconfigPath = providedDocgenConfig.tsconfigPath || paths_1.paths.appTsConfig;
    const { config } = ts.readConfigFile(tsconfigPath, (path) => fs.readFileSync(path, 'utf8'));
    const theme = {
        ...defaultThemeConfiguration,
        ...providedThemeConfig,
    };
    const basePath = options.basePath ||
        (providedBuildConfig.basePath
            ? (0, core_1.removeTrailingSlash)(providedBuildConfig.basePath)
            : defaultConfig.basePath);
    const assetDir = providedConfig.assetDir && (0, paths_1.resolveApp)(providedConfig.assetDir);
    _normalizedConfig = {
        ...defaultConfig,
        ...providedConfig,
        componentsEntry,
        example: {
            placeholder: placeholder && (0, paths_1.resolveApp)(placeholder),
            dimensions: normalizeDimensions(dimensions),
            enableAdvancedEditor,
            syncStateType,
            showDeviceFrame,
            a11y,
        },
        html,
        sass,
        css: {
            enabled: !!css,
            usePostcss: !!(css && css.postcss),
        },
        ignores,
        sections,
        basePath,
        assetDir,
        wrapper: providedConfig.wrapper && (0, paths_1.resolveApp)(providedConfig.wrapper),
        cacheDir: (0, paths_1.resolveApp)(cacheDir),
        outDir: (0, core_1.isString)(options.outDir) ? options.outDir : outDir,
        prerender,
        devServerPort: providedDevServerConfig.port || 6969,
        docgen: {
            tsconfigPath,
            options: docgenOptions,
        },
        theme: {
            ...theme,
            manifest: providedManifestConfig && {
                name: theme.title,
                short_name: theme.title,
                background_color: '#ffffff',
                theme_color: theme.colors['primary-500'],
                start_url: `${basePath}/`,
                display: 'standalone',
                ...providedManifestConfig,
                icons: providedManifestConfig.icons ||
                    (function getIcons() {
                        const favIconPath = providedManifestConfig.baseIconPath &&
                            (0, paths_1.resolveApp)(providedManifestConfig.baseIconPath);
                        if (favIconPath && fs.existsSync(favIconPath)) {
                            const iconMime = mimeTypes.lookup(favIconPath);
                            if (iconMime) {
                                return [
                                    {
                                        src: `${basePath}/_icons/${path.parse(favIconPath).base}`,
                                        type: iconMime,
                                        sizes: 'any',
                                    },
                                ];
                            }
                        }
                        return [];
                    })(),
            },
        },
        imports: componentsEntry ? imports.concat(componentsEntry) : imports,
        search: {
            includeHeadings: searchIncludeHeadings,
        },
        compilerOptions: (config && config.compilerOptions) || {},
    };
    return _normalizedConfig;
    function collectComponents(componentPaths, parent, parentSlugs, hideFromSidebar) {
        componentPaths.forEach((comPath) => {
            const comPathInfo = path.parse(comPath);
            let docPath = null;
            for (const ext of COMPONENT_DOC_EXTENSIONS) {
                const possibleDocPath = `${comPathInfo.dir}/${comPathInfo.name}${ext}`;
                if (fs.existsSync(possibleDocPath)) {
                    docPath = possibleDocPath;
                    break;
                }
            }
            const section = {
                type: 'component',
                sourcePath: comPath,
                docPath,
                parentSlugs,
                id: (0, create_hash_1.createHash)(comPath),
                hideFromSidebar,
            };
            components.push(section);
            parent.push(section);
        });
    }
    function collectSections(sectionConfigs, parent, parentSlugs) {
        sectionConfigs.forEach((sectionConfig) => {
            switch (sectionConfig.type) {
                case 'group': {
                    const title = sectionConfig.title;
                    if (!title) {
                        // if no title, this is a empty group, so delegate to parent
                        collectSections(sectionConfig.items, parent, sectionConfig.path
                            ? parentSlugs.concat(sectionConfig.path)
                            : parentSlugs);
                        return;
                    }
                    const slug = sectionConfig.path || (0, slugify_1.default)(title, { lower: true });
                    if (slug.startsWith('_')) {
                        (0, log_to_stdout_1.logToStdout)((0, log_to_stdout_1.yellow)('Having path starts with _ may causes unexpected behavior.'));
                        (0, log_to_stdout_1.logToStdout)(`Path is "${slug}" for ${title}`);
                    }
                    const section = {
                        type: 'group',
                        title,
                        slug: parentSlugs.concat(slug).join('/'),
                        hideFromSidebar: sectionConfig.hideFromSidebar,
                        items: [],
                    };
                    collectSections(sectionConfig.items, section.items, parentSlugs.concat(slug));
                    parent.push(section);
                    return;
                }
                case 'components': {
                    const title = sectionConfig.title;
                    const componentPaths = Array.isArray(sectionConfig.components)
                        ? (0, core_1.flattenArray)(sectionConfig.components.map((pattern) => glob.sync(pattern, {
                            cwd: paths_1.paths.appPath,
                            absolute: true,
                            ignore: ignores,
                        })))
                        : glob.sync(sectionConfig.components, {
                            cwd: paths_1.paths.appPath,
                            absolute: true,
                            ignore: ignores,
                        });
                    if (componentPaths.length === 0) {
                        return;
                    }
                    if (!title) {
                        collectComponents(componentPaths, parent, sectionConfig.path
                            ? parentSlugs.concat(sectionConfig.path)
                            : parentSlugs, sectionConfig.hideFromSidebar);
                        return;
                    }
                    const slug = sectionConfig.path || (0, slugify_1.default)(title, { lower: true });
                    if (slug.startsWith('_')) {
                        (0, log_to_stdout_1.logToStdout)((0, log_to_stdout_1.yellow)('Having path starts with _ may causes unexpected behavior.'));
                        (0, log_to_stdout_1.logToStdout)(`Path is "${slug}" for ${title}`);
                    }
                    const section = {
                        type: 'group',
                        title,
                        slug: parentSlugs.concat(slug).join('/'),
                        items: [],
                        hideFromSidebar: sectionConfig.hideFromSidebar,
                    };
                    collectComponents(componentPaths, section.items, parentSlugs.concat(slug), sectionConfig.hideFromSidebar);
                    parent.push(section);
                    return;
                }
                case 'content': {
                    const docPath = path.resolve(paths_1.paths.appPath, sectionConfig.content);
                    if (!fs.existsSync(docPath)) {
                        return;
                    }
                    const slug = (0, core_1.isString)(sectionConfig.path)
                        ? sectionConfig.path
                        : (sectionConfig.title &&
                            (0, slugify_1.default)(sectionConfig.title, { lower: true })) ||
                            path.parse(docPath).name;
                    if (slug.startsWith('_')) {
                        (0, log_to_stdout_1.logToStdout)((0, log_to_stdout_1.yellow)('Having path starts with _ may causes unexpected behavior.'));
                        (0, log_to_stdout_1.logToStdout)(`Path is "${slug}" for ${sectionConfig.content}`);
                    }
                    parent.push({
                        type: 'markdown',
                        title: sectionConfig.title,
                        sourcePath: docPath,
                        slug: parentSlugs.concat(slug).join('/'),
                        hideFromSidebar: sectionConfig.hideFromSidebar,
                        formatLabel: (x) => x,
                        _associatedComponentName: sectionConfig._associatedComponentName,
                    });
                    return;
                }
                case 'link': {
                    parent.push(sectionConfig);
                    return;
                }
                case 'docs': {
                    const docsFolder = sectionConfig.folder;
                    const docGroupTitle = sectionConfig.title;
                    const formatLabel = sectionConfig.formatLabel || ((x) => x);
                    const pagesPaths = glob.sync(`${docsFolder}/**/*.{md,mdx}`, {
                        cwd: paths_1.paths.appPath,
                        ignore: sectionConfig.ignores,
                    });
                    if (docGroupTitle) {
                        const docPathPrefix = sectionConfig.path || (0, slugify_1.default)(docGroupTitle, { lower: true });
                        const slugParts = parentSlugs.concat(docPathPrefix);
                        const section = {
                            type: 'group',
                            title: docGroupTitle,
                            slug: slugParts.join('/'),
                            items: [],
                            hideFromSidebar: sectionConfig.hideFromSidebar,
                        };
                        collectDocs(section.items, slugParts, formatLabel);
                        parent.push(section);
                    }
                    else {
                        collectDocs(parent, parentSlugs, formatLabel);
                    }
                    function collectDocs(targetItems, pathToDoc, formatLabel) {
                        pagesPaths.forEach((pagePath) => {
                            const pathInfo = path.parse(pagePath);
                            const slug = pathInfo.name === 'index' ? '' : pathInfo.name;
                            if (slug.startsWith('_')) {
                                (0, log_to_stdout_1.logToStdout)((0, log_to_stdout_1.yellow)('Having path starts with _ may causes unexpected behavior.'));
                                (0, log_to_stdout_1.logToStdout)(`Path is "${slug}" for ${pagePath}`);
                            }
                            targetItems.push({
                                type: 'markdown',
                                sourcePath: path.resolve(paths_1.paths.appPath, pagePath),
                                slug: pathToDoc.concat(slug).join('/'),
                                formatLabel,
                            });
                        });
                    }
                    return;
                }
                default:
                    return;
            }
        });
    }
};
exports.getConfig = getConfig;
function normalizeDimensions(dimensions) {
    const result = [];
    dimensions.forEach((d) => {
        if ((0, core_1.isString)(d)) {
            if (core_1.deviceDimensionsByName[d]) {
                result.push(core_1.deviceDimensionsByName[d]);
            }
            else {
                (0, log_to_stdout_1.logToStdout)(`Invalid device preset: ${d}`);
            }
        }
        else {
            const { width, height, name = String(width) } = d;
            result.push({
                width,
                height,
                name,
            });
        }
    });
    return result;
}
const getUserConfig = (env, configFile) => {
    const configFilePath = configFile
        ? (0, paths_1.resolveApp)(configFile)
        : paths_1.paths.appShowroomConfig;
    if (!fs.existsSync(configFilePath)) {
        return {};
    }
    const provided = require(configFilePath);
    return typeof provided === 'function'
        ? provided(env === 'development' ? 'dev' : 'build')
        : provided;
};
const COMPONENT_DOC_EXTENSIONS = ['.mdx', '.md'];
const isSassInstalled = () => {
    try {
        require.resolve('sass', {
            paths: [paths_1.paths.appPath],
        });
        return true;
    }
    catch (err) {
        return false;
    }
};
//# sourceMappingURL=get-config.js.map